/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <ipc/thread.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "accessing.h"
#include "memory_incremental.h"
#include "memory_utils.h"
#include "page_queue_shared.h"
#include "pages.h"
#include "paging.h"
#include "resource_server.h"
#include "test_file_opener.h"



const unsigned int REGION_PAGES = 8;
const unsigned int MEMORY_PAGES = REGION_PAGES * 10;
const unsigned int FILE_PAGES = 20;

int main(int argc, char *argv[])
{
  long err;

  /* Introduce concurrency control. */

  err = ipc_thread_init();

  if (err)
  {
    printf("Initialisation error: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Configure the number of available pages using any argument. */

  unsigned int memory_pages = MEMORY_PAGES;

  if (argc > 1)
    memory_pages = atoi(argv[1]) * REGION_PAGES;

  /* Some memory plus infrastructure. */

  MemoryIncremental mem(memory_pages, page(REGION_PAGES));
  Accessing accessing;
  Paging paging;
  PageQueueShared queue;
  Pages pages(&mem, &queue);
  TestFileOpener opener(&accessing, &paging, &pages, page(FILE_PAGES));

  /* Register a server associating it with the given object. */

  ResourceServer server(&opener);
  err = server.bind("server");

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server...\n");
  server.start();
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
